module namespace wheel = "http://wheelfactory.com";

import module namespace util = "http://exist-db.org/xquery/util";
import module namespace xmldb = "http://exist-db.org/xquery/xmldb";

declare namespace rest="http://exquery.org/ns/rest/annotation/";
declare namespace output = "http://www.w3.org/2010/xslt-xquery-serialization";

declare
    %rest:POST("{$wheel-doc}")
    %rest:path("/factory/warehouse/wheel")
function wheel:create-wheel($wheel-doc as document-node()) {
    let $id := util:uuid() return
        if(xmldb:store("/db/wheels", (),
            <wheel id="{$id}" created="{current-dateTime()}">
                { $wheel-doc/wheel/* }
            </wheel>
        ))then
            <a href="">{$id}</a>
        else()
};

declare
    %rest:PUT("{$wheel-doc}")
    %rest:path("/factory/warehouse/wheel/{$id}")
function wheel:store-wheel($id, $wheel-doc as document-node()) as xs:boolean {
    let $existing-wheel := wheel:retrieve-wheel($id) return
        let $filename :=
            if(not(empty($existing-wheel)))then
              document-uri(root($existing-wheel))  
            else()
        return
            not(empty(
                xmldb:store("/db/wheels", $filename,
                    <wheel id="{$id}" created="{current-dateTime()}">
                        { $wheel-doc/wheel/* }
                    </wheel>
                )
            ))
};

declare
    %rest:GET
    %rest:path("/factory/warehouse/wheel")
function wheel:inventory() as element(inventory) {
    <inventory>
    {
        for $wheel in collection("/db/wheels")/wheel return
            <wheel id="{$wheel/@id}" created="{$wheel/@created}"/>
    }
    </inventory>
};

declare
    %rest:GET
    %rest:path("/factory/warehouse/wheel/{$id}")
function wheel:retrieve-wheel($id) {
    let $wheel := collection("/db/wheels")/wheel[@id eq $id] return
        if(empty($wheel))then
            wheel:not-found()
        else
            $wheel
};

declare function wheel:not-found() {
   ( 
      <rest:response>
           <output:serialization-parameters>
                <output:method value="binary"/>
           </output:serialization-parameters>
           <http:response status="404" reason="not found">
               <http:header name="Content-Type" value="image/jpeg"/>
           </http:response>
       </rest:response>,
       util:binary-doc("/db/examples/404.jpg")
    )
};